function Phi_next = acwe_iter_color(Phi, I_color, epsilon, mu, nu, lambda_1, lambda_2,...
    dt, edgevectors, lengths, angles, omegas, right_lengths, index, n, al,...
    degs, fil, curv_approx)

% Inputs:
%   Phi             - the embedding function in the previous step
%   dt              - time step determining to which extent the embedding
%                   function is updated
%   edgevectors     - mx2xn matrix (m = maximal vertex degree) each slice
%                   of which contains the unit edge vectors starting from
%                   the corresponding vertex in its first rows                     
%   lengths         - mx1xn matrix containing the edges' lengths
%                   **modified** so that edges with zero length are
%                   assigned a unit length, for proper division
%   angles          - mx1xn matrix holding in each slice the angles 
%                   "occupied" by the corresponding vertex's neighbors
%   omegas          - mx1xn matrix holding in each slice the angles 
%                   assigned to each neighbor of the corresponding vertex
%   right_lengths   - mx1xn matrix containing the actual edges' lengths
%   index           - mx1xn matrix holding in each slice the indices of the
%                   adjacency list of the corresponding vertex in an order
%                   such that the neighbors are sorted in ascending order
%                   w.r.t. the edge's angle with the x-axis
%   n               - number of graph vertices
%   al              - adjacency list of the graph stored as an nxm matrix,
%                   where m is maximal vertex degree on the graph          
%   degs            - 1xn vector holding the degree of each vertex
%   fil             - 1x2 vector controlling the type of filter (average or
%                   median) used to smooth the gradient and curvature
%                   approximations
%   curv_approx     - parameter determining the kind of approximation used
%                   to compute the curvature; differential or integral

% Outputs:
%   Phi_next        - the embedding function after the current step

% Optimize average RGB values inside and outside contour.
c_1 = mean(I_color(Phi >= 0, :));
c_2 = mean(I_color(Phi < 0, :));

% Gradient computation

[gr_Phi, grmag_Phi] = gradient_approximation_step(n, al, degs, Phi,...
    edgevectors, lengths, angles);

if (fil(1) == 1)
    % Average filtering
    [gr_Phi, grmag_Phi] = gradient_average_filter(al, degs, n, gr_Phi);
        
else if (fil(1) == 2)
        % Median filtering
        [gr_Phi, grmag_Phi] = gradient_median_filter(al, degs, n, gr_Phi);
    end
end

% Curvature computation

if curv_approx
    % Method 2 for curvature - Geometric approximation
    curv_Phi = curv_geom(al, degs, n, angles, omegas, right_lengths, index,...
        gr_Phi, grmag_Phi, fil(2));
else
    % Method 1 for curvature - Gradient-based approximation
    curv_Phi = curv_sum_pds(n, al, degs, gr_Phi, grmag_Phi,...
        edgevectors, lengths, angles, fil(2));
end

% Regularized Dirac delta function of Phi.
delta_epsilon = (epsilon / pi) ./ (epsilon ^ 2 + Phi .^ 2);

% ACWE partial difference equation

% Update embedding function Phi to produce output Phi_next according to the
% partial difference equation constituting the ACWE algorithm

Phi_next = Phi + dt * delta_epsilon .* (mu * curv_Phi - nu...
    - lambda_1 * sum((I_color - repmat(c_1, n, 1)) .^ 2, 2)...
    + lambda_2 * sum((I_color - repmat(c_2, n, 1)) .^ 2, 2));

% Median filtering

% Filter the updated embedding function with median filter to smooth the
% result
% Phi_next = median_filter(al, degs, n, Phi_next);

end

