function c =...
    curv_geom(al, degs, n, angles, omegas, right_lengths, index, grad, gradmagn, fil)

% Inputs:
%   al              - adjacency list of the graph stored as an nxm matrix,
%                   where m is maximal vertex degree on the graph
%   degs            - 1xn vector holding the degree of each vertex
%   n               - number of graph vertices
%   angles          - mx1xn matrix holding in each slice the angles 
%                   "occupied" by the corresponding vertex's neighbors
%   omegas          - mx1xn matrix holding in each slice the angles 
%                   assigned to each neighbor of the corresponding vertex
%   right_lengths   - mx1xn matrix containing the actual edges' lengths
%   index           - mx1xn matrix holding in each slice the indices of the
%                   adjacency list of the corresponding vertex in an order
%                   such that the neighbors are sorted in ascending order
%                   w.r.t. the edge's angle with the x-axis
%   grad            - nx2 matrix with approximate gradient field
%   gradmagn        - nx1 vector with magnitude of gradient field
%   fil             - parameter controlling the type of filter used to
%                   smooth the curvature approximation

% Outputs:
%   c           - nx1 vector holding the approximate curvature of the
%               embedding function's levelsets obtained with integral-based
%               approximation

c = zeros(n, 1);

% Gradient magnitude modification to avoid ill 0/0 fractions in F field
% calculation
epsilon = 10^(-6);
gradmagn(gradmagn < epsilon) = epsilon;

% Unit gradient vector field (F) calculation
F = [grad(:,1)./gradmagn, grad(:,2)./gradmagn];

% Calculation of the total area of S(v) for each vertex v in the column
% vector areas
areas = sum((angles/2).*(right_lengths.^2));
areas = permute(areas, [3 2 1]);

for i=1:n
    field = F(al(i, index(1:degs(i), 1, i)), :);
    
    % Calculation of trigonometric functions' differences
    trigs = zeros(degs(i), 2);
    trigs(1:end-1, 1) = sin(omegas(index(2:degs(i), 1, i), 1, i)) -...
        sin(omegas(index(1:degs(i)-1, 1, i), 1, i));
    trigs(1:end-1, 2) = -cos(omegas(index(2:degs(i), 1, i), 1, i)) +...
        cos(omegas(index(1:degs(i)-1, 1, i), 1, i));
    if (degs(i) > 0)
        trigs(end, 1) = sin(omegas(index(1, 1, i), 1, i)) -...
            sin(omegas(index(degs(i), 1, i), 1, i));
        trigs(end, 2) = -cos(omegas(index(1, 1, i), 1, i)) +...
            cos(omegas(index(degs(i), 1, i), 1, i));
    end
    
    dists = right_lengths(index(1:degs(i), 1, i), 1, i);
    
    % Contribution of field "integral" over the circular arcs comprising
    % the boundary of S(v)
    arcs = sum(dists.*sum(field.*trigs, 2));
    
    % Calculation of the field on the linear segments comprising the
    % boundary of S(v)
    sum_next = zeros(degs(i), 2);
    sum_next(1:end-1, :) = field(1:end-1, :) + field(2:end, :);
    if (degs(i) > 0)
        sum_next(end, :) = field(end, :) + field(1, :);
    end
    magns = sqrt(sum_next(:, 1).^2 + sum_next(:, 2).^2);
    % Modification to avoid ill 0/0 fractions
    magns(magns < epsilon) = epsilon;
    sum_next_norm = sum_next./[magns, magns];
    
    trigs2 = zeros(degs(i), 2);
    trigs2(1:end-1, 1) = sin(omegas(index(2:degs(i), 1, i), 1, i));
    trigs2(1:end-1, 2) = -cos(omegas(index(2:degs(i), 1, i), 1, i));
    if (degs(i) > 0)
        trigs2(end, 1) = sin(omegas(index(1, 1, i), 1, i));
        trigs2(end, 2) = -cos(omegas(index(1, 1, i), 1, i));
    end
    
    dists_diffs = zeros(degs(i), 1);
    dists_diffs(1:end-1) = dists(2:end)-dists(1:end-1);
    if (degs(i) > 0)
        dists_diffs(end) = dists(1) - dists(end);
    end
    
    % Contribution of field "integral" over the linear segments of the
    % boundary
    linsegs = sum(dists_diffs.*sum(sum_next_norm.*trigs2, 2));
    
    % Curvature at vertex v as sum of contributions of arcs and linear
    % segments divided by the area of S(v)
    if areas(i) ~= 0
        c(i) = (arcs + linsegs)/areas(i);
    else
        % In case of zero area (no neighbour), assign zero to curvature(?)
        c(i) = 0;
    end
end

% Optionary filtering of curvature approximation
if (fil == 1)
    % Average filtering
    degs = degs + 1;
    al = [al zeros(n, 1)];
    for i=1:n
        al(i, degs(i)) = i;
    end
    
    % Main average filtering loop
    for i=1:n
        c(i) = mean(c(al(i, 1:degs(i))));
    end
    
else if (fil == 2)
        % Median filtering
        degs = degs + 1;
        al = [al zeros(n, 1)];
        for i=1:n
            al(i, degs(i)) = i;
        end
        
        % Main median filtering loop
        for i=1:n
            c(i) = median(c(al(i, 1:degs(i))));
        end
    end    
end

end

