function [edgevectors, lengths, right_lengths, angles, omegas, index] =...
    edgevectors_angles(n, al, c, degs)

% Inputs:
%   n       - number of graph vertices
%   al      - adjacency list of the graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   c       - nx2 matrix holding the positions of the vertices
%   degs    - 1xn vector holding the degree of each vertex

% Outputs:
%   edgevectors     - mx2xn matrix (m = maximal vertex degree) each slice
%                   of which contains the unit edge vectors starting from
%                   the corresponding vertex in its first rows
%   lengths         - mx1xn matrix containing the edges' lengths
%                   **modified** so that edges with zero length are
%                   assigned a unit length, for proper division
%   right_lengths   - mx1xn matrix containing the actual edges' lengths
%   angles          - mx1xn matrix holding in each slice the angles 
%                   "occupied" by the corresponding vertex's neighbors
%   omegas          - mx1xn matrix holding in each slice the angles 
%                   assigned to each neighbor of the corresponding vertex
%   index           - mx1xn matrix holding in each slice the indices of the
%                   adjacency list of the corresponding vertex in an order
%                   such that the neighbors are sorted in ascending order
%                   w.r.t. the edge's angle with the x-axis

m = max(degs);

% Defining the unit vectors, e-vw, corresponding to the edges of the graph, 
% with their origins at the reference vertex, and the distances between the 
% vertices, d(w,v), which correspond to the lengths of the edge vectors

edgevectors = zeros(m, 2, n);
for i=1:n
    edgevectors(1:degs(i), :, i) = c(al(i, 1:degs(i)), :) - ones(degs(i), 1)*c(i, :);
end

lengths = sqrt(edgevectors(:, 1, :).^2 + edgevectors(:, 2, :).^2);
% Avoid dividing edgevectors by zero length, without altering the results
noedges = (lengths == 0);
lengths = lengths + noedges;
edgevectors(:, 1, :) = edgevectors(:, 1, :)./lengths;
edgevectors(:, 2, :) = edgevectors(:, 2, :)./lengths;
right_lengths = lengths - noedges;

% Defining the angle w which is "occupied" by each edge vw incident to v

phis = atan2(edgevectors(:, 2, :), edgevectors(:, 1, :));
angles = zeros(m, 1, n);
omegas = zeros(m, 1, n);
index = zeros(m, 1, n);

for i=1:n
    [sortedphis, indices] = sort(phis(1:degs(i), 1, i));
    % Fill the index of neighbours' ordering according to edge angle
    index(1:degs(i), 1, i) = indices;
    % Computing the angle for all edges but "first" and "last"
    angles(indices(2:end-1), 1, i) = (sortedphis(3:end) - sortedphis(1:end-2))/2;
    omegas(indices(2:end), 1, i) = (sortedphis(2:end) + sortedphis(1:end-1))/2;
    % "First" edge needs special treatment, so does "last"
    if (size(sortedphis, 1) >= 2)
        angles(indices(1), 1, i) = (sortedphis(2) - (sortedphis(end) - 2*pi))/2;
        angles(indices(end), 1, i) = ((sortedphis(1) + 2*pi) - sortedphis(end-1))/2;
        omegas(indices(1), 1, i) = (sortedphis(1) + sortedphis(end) - 2*pi)/2;
    else if (size(sortedphis, 1) == 1)
        % In the case of only one neighbour, we define w = pi
        angles(1, 1, i) = pi;
        omegas(1, 1, i) = sortedphis(1) + pi;
        end
    end
end

end

