function [g, I_s_grmag] = g_function_gauss_grad_sep_nrmlz(I, c, n, al, degs, sigma, lambda)

% Inputs:
%   I       - initial intensity function on the graph vertices
%   c       - nx2 matrix holding the positions of the vertices
%   n       - number of graph vertices
%   al      - adjacency list of the graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   degs    - 1xn vector holding the degree of each vertex
%   sigma   - scale of the graph-theoretic Gaussian smoothing filter
%   lambda  - scale parameter in formula for computation of g function

% Outputs:
%   g           - nx1 vector holding the values of g on the graph's
%               vertices
%   I_s_grmag   - nx1 vector with gradient magnitude of the smoothed
%               intensity function

% Compute distance of every pair of nodes in the graph in distances matrix
[xs1, xs2] = meshgrid(c(:, 1), c(:, 1));
[ys1, ys2] = meshgrid(c(:, 2), c(:, 2));
squared_distances = (xs1 - xs2).^2 + (ys1 - ys2).^2;

% Gaussian gradient smoothing function
gauss = exp(-squared_distances/(2*sigma^2))/(2*pi*sigma^2);
clear squared_distances;

% Gaussian derivatives
gauss_x = ((xs2 - xs1).*gauss)/(sigma^2);
clear xs1 xs2;
gauss_y = ((ys2 - ys1).*gauss)/(sigma^2);
clear ys1 ys2 gauss;

% Separate positive from negative part
gauss_x_pos = max(gauss_x, 0);
gauss_x_neg = min(gauss_x, 0);
clear gauss_x;
gauss_y_pos = max(gauss_y, 0);
gauss_y_neg = min(gauss_y, 0);
clear gauss_y;

% Take care of 0/0 cases
sum_gauss_x_pos = sum(gauss_x_pos, 2);
sum_gauss_x_pos(sum_gauss_x_pos == 0) = eps;
sum_gauss_x_neg = sum(gauss_x_neg, 2);
sum_gauss_x_neg(sum_gauss_x_neg == 0) = -eps;
sum_gauss_y_pos = sum(gauss_y_pos, 2);
sum_gauss_y_pos(sum_gauss_y_pos == 0) = eps;
sum_gauss_y_neg = sum(gauss_y_neg, 2);
sum_gauss_y_neg(sum_gauss_y_neg == 0) = -eps;

% Compute smoothed image gradient with separated normalization
I_s_x = (gauss_x_pos*I)./sum_gauss_x_pos - (gauss_x_neg*I)./sum_gauss_x_neg;
I_s_y = (gauss_y_pos*I)./sum_gauss_y_pos - (gauss_y_neg*I)./sum_gauss_y_neg;
I_s_grmag = sqrt(I_s_x.^2 + I_s_y.^2);

% Filter the results with median filter to reduce the error
I_s_grmag = median_filter(al, degs, n, I_s_grmag);

% Compute g function with smoothed image gradient as its argument using
% analytical form in Drakopoulos' - Maragos' paper (Caselles-Catte-Coll-
% Dibos and Malladi-Sethian-Vemuri proposal)
g = (1 + (I_s_grmag.^2)/(lambda^2)).^(-1);

end

