function [g, I_s_grmag, I_s] = g_function_maxapprox_grad(I, c, n, al, degs, sigma, lambda)

% Inputs:
%   I       - initial intensity function on the graph vertices
%   c       - nx2 matrix holding the positions of the vertices
%   n       - number of graph vertices
%   al      - adjacency list of the graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   degs    - 1xn vector holding the degree of each vertex
%   sigma   - scale of the graph-theoretic Gaussian smoothing filter
%   lambda  - scale parameter in formula for computation of g function

% Outputs:
%   g           - nx1 vector holding the values of g on the graph's
%               vertices
%   I_s_grmag   - nx1 vector with gradient magnitude of the smoothed
%               intensity function
%   I_s         - nx1 vector with smoothed intensity function

d = size(c, 2);

% Compute distance of every pair of nodes in the graph in distances matrix
tmp_1 = zeros(n, 3 * d);
tmp_2 = zeros(n, 3 * d);
for i = 1:d
    tmp_1(:, 3*i-2:3*i) = [ones(n, 1), -2 * c(:, i), c(:, i) .^ 2];
    tmp_2(:, 3*i-2:3*i) = [c(:, i) .^ 2 , c(:, i), ones(n, 1)];
end
squared_distances = tmp_1 * tmp_2.';

% Gaussian gradient smoothing function
gauss = exp(-squared_distances/(2*sigma^2))/(2*pi*sigma^2);

% Compute smoothed image
I_s = (gauss*I)./sum(gauss, 2);

% Compute smoothed image's gradient with old faithful gradient approximation
I_s_grmag = gradmagn_max(n, al, degs, I_s);

% Filter the results with median filter to reduce the error
I_s_grmag = median_filter(al, degs, n, I_s_grmag);

% Compute g function with smoothed image gradient as its argument using
% analytical form in Drakopoulos' - Maragos' paper (Caselles-Catte-Coll-
% Dibos and Malladi-Sethian-Vemuri proposal)
g = (1 + (I_s_grmag.^2)/(lambda^2)).^(-1);

end