function [L, u, indices, am, al, c, degs, del, n] = graph_watershed_delaunay(I)

% Inputs:
%   I       - grayscale image on which the graph and its intensity function
%           will be defined

% Outputs:
%   L       - watershed labeling of image I
%   u       - intensity function on the graph 
%   indices - linear indices of image pixels assigned to graph vertices
%   am      - nxn adjacency matrix of the created graph
%   al      - adjacency list of the created graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   c       - nx2 matrix holding the positions of the vertices
%   degs    - 1xn vector holding the degree of each vertex
%   del     - tx3 matrix holding the triples of vertices corresponding
%   n       - number of graph vertices - equal to number of watershed
%           basins

[height, width] = size(I);

% Gradient calculation using Sobel operator
Sx = [1 2 1]' * [-1 0 1]; Sy = Sx'; % Sobel operator
Gx = imfilter(I, Sx/8, 'replicate');
Gy = imfilter(I, Sy/8, 'replicate');
Gm = sqrt(Gx.^2 + Gy.^2); % Image's gradient magnitude

% Apply watershed transform directly on image gradient to achieve the
% desired oversegmentation
L = watershed(Gm);

% Number of different regions
regions_count = max(L(:));
c = [];

% Graph vertices as ultimate erosions of watershed regions
for i=1:regions_count
    region = (L == i);
    D = bwdist(~region);
    [~, inds] = max(D(:));
    % Assign only one vertex to each region
    if (numel(inds) >= 2)
        inds = inds(1);
    end
    [is, js] = ind2sub([height, width], inds);
    c = [c; [js is]];
end

% Number of graph vertices
n = size(c, 1);

del = delaunay(c);
[triangles, ~] = size(del);

am = zeros(n);
al = zeros(n);
degs = zeros(1, n);
for k=1:triangles
    if (am(del(k, 1), del(k, 2)) == 0)
        am(del(k, 1), del(k, 2)) = 1;
        am(del(k, 2), del(k, 1)) = 1;
        degs(del(k, 1)) = degs(del(k, 1)) + 1;
        degs(del(k, 2)) = degs(del(k, 2)) + 1;
        al(del(k, 1), degs(del(k, 1))) = del(k, 2);
        al(del(k, 2), degs(del(k, 2))) = del(k, 1);
    end
    if (am(del(k, 1), del(k, 3)) == 0)
        am(del(k, 1), del(k, 3)) = 1;
        am(del(k, 3), del(k, 1)) = 1;
        degs(del(k, 1)) = degs(del(k, 1)) + 1;
        degs(del(k, 3)) = degs(del(k, 3)) + 1;
        al(del(k, 1), degs(del(k, 1))) = del(k, 3);
        al(del(k, 3), degs(del(k, 3))) = del(k, 1);
    end
    if (am(del(k, 2), del(k, 3)) == 0)
        am(del(k, 2), del(k, 3)) = 1;
        am(del(k, 3), del(k, 2)) = 1;
        degs(del(k, 2)) = degs(del(k, 2)) + 1;
        degs(del(k, 3)) = degs(del(k, 3)) + 1;
        al(del(k, 2), degs(del(k, 2))) = del(k, 3);
        al(del(k, 3), degs(del(k, 3))) = del(k, 2);
    end
end

al = al(:, 1:max(degs));

% Defining the gray-scale image function on the graph's vertices

indices = sub2ind([height, width], c(:, 2), c(:, 1));
u = I(indices);

end
