function [u, indices, am, al, c, degs, del, n] = im2graph_delaunay(n, image)

% Inputs:
%   n       - number of **initial** graph vertices
%   image   - grayscale image on which the graph and its intensity function
%           will be defined

% Outputs:
%   u       - intensity function on the graph 
%   indices - linear indices of image pixels assigned to graph vertices
%   am      - nxn adjacency matrix of the created graph
%   al      - adjacency list of the created graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   c       - nx4 matrix holding the positions of the vertices (continuous
%           in the first 2 columns and pixelwise in the last 2)
%   degs    - 1xn vector holding the degree of each vertex
%   del     - tx3 matrix holding the triples of vertices corresponding
%           to the faces of the Delaunay triangulation
%   n       - final number of graph vertices after elimination

[height, width] = size(image);

coords = [width*rand(n, 1), height*rand(n, 1)];

% Assigning vertices of the graph to pixels of the image

pixelcoords = zeros(n, 2);
pixelcoords(:, 1) = ceil(coords(:, 1));
pixelcoords(:, 2) = ceil(coords(:, 2));

% Rejecting all redundant vertices by keeping at most one vertex per pixel

[~, uniquevert, ~] = unique(pixelcoords, 'rows');
uniquevert = sort(uniquevert);
n = length(uniquevert);
c = [coords(uniquevert, :) pixelcoords(uniquevert, :)];

del = delaunay(c(:, 1:2));
[triangles, ~] = size(del);

am = zeros(n);
al = zeros(n);
degs = zeros(1, n);
for k=1:triangles
    if (am(del(k, 1), del(k, 2)) == 0)
        am(del(k, 1), del(k, 2)) = 1;
        am(del(k, 2), del(k, 1)) = 1;
        degs(del(k, 1)) = degs(del(k, 1)) + 1;
        degs(del(k, 2)) = degs(del(k, 2)) + 1;
        al(del(k, 1), degs(del(k, 1))) = del(k, 2);
        al(del(k, 2), degs(del(k, 2))) = del(k, 1);
    end
    if (am(del(k, 1), del(k, 3)) == 0)
        am(del(k, 1), del(k, 3)) = 1;
        am(del(k, 3), del(k, 1)) = 1;
        degs(del(k, 1)) = degs(del(k, 1)) + 1;
        degs(del(k, 3)) = degs(del(k, 3)) + 1;
        al(del(k, 1), degs(del(k, 1))) = del(k, 3);
        al(del(k, 3), degs(del(k, 3))) = del(k, 1);
    end
    if (am(del(k, 2), del(k, 3)) == 0)
        am(del(k, 2), del(k, 3)) = 1;
        am(del(k, 3), del(k, 2)) = 1;
        degs(del(k, 2)) = degs(del(k, 2)) + 1;
        degs(del(k, 3)) = degs(del(k, 3)) + 1;
        al(del(k, 2), degs(del(k, 2))) = del(k, 3);
        al(del(k, 3), degs(del(k, 3))) = del(k, 2);
    end
end

al = al(:, 1:max(degs));

% Defining the gray-scale image function on the graph's vertices

indices = sub2ind([height, width], c(:, 4), c(:, 3));
u = image(indices);

end
